@@ -4,7 +4,7 @@ use crate::{
     state::{
         enums::GovernanceAccountType,
         governance::{
-            assert_is_valid_governance_config, get_mint_governance_address_seeds, Governance,
+            assert_valid_create_governance_args, get_mint_governance_address_seeds, Governance,
             GovernanceConfig,
         },
     },
@@ -44,11 +44,13 @@ pub fn process_create_mint_governance(
     let rent_sysvar_info = next_account_info(account_info_iter)?; // 7
     let rent = &Rent::from_account_info(rent_sysvar_info)?;
 
-    assert_is_valid_governance_config(program_id, &config, realm_info)?;
+    assert_valid_create_governance_args(program_id, &config, realm_info)?;
 
     let mint_governance_data = Governance {
         account_type: GovernanceAccountType::MintGovernance,
-        config: config.clone(),
+        realm: *realm_info.key,
+        governed_account: *governed_mint_info.key,
+        config,
         proposals_count: 0,
         reserved: [0; 8],
     };
@@ -57,7 +59,7 @@ pub fn process_create_mint_governance(
         payer_info,
         mint_governance_info,
         &mint_governance_data,
-        &get_mint_governance_address_seeds(&config.realm, &config.governed_account),
+        &get_mint_governance_address_seeds(realm_info.key, governed_mint_info.key),
         program_id,
         system_info,
         rent,
