@@ -28,13 +28,14 @@ pub fn process_create_realm(
     let account_info_iter = &mut accounts.iter();
 
     let realm_info = next_account_info(account_info_iter)?; // 0
-    let governance_token_mint_info = next_account_info(account_info_iter)?; // 1
-    let governance_token_holding_info = next_account_info(account_info_iter)?; // 2
-    let payer_info = next_account_info(account_info_iter)?; // 3
-    let system_info = next_account_info(account_info_iter)?; // 4
-    let spl_token_info = next_account_info(account_info_iter)?; // 5
+    let realm_authority_info = next_account_info(account_info_iter)?; // 1
+    let governance_token_mint_info = next_account_info(account_info_iter)?; // 2
+    let governance_token_holding_info = next_account_info(account_info_iter)?; // 3
+    let payer_info = next_account_info(account_info_iter)?; // 4
+    let system_info = next_account_info(account_info_iter)?; // 5
+    let spl_token_info = next_account_info(account_info_iter)?; // 6
 
-    let rent_sysvar_info = next_account_info(account_info_iter)?; // 6
+    let rent_sysvar_info = next_account_info(account_info_iter)?; // 7
     let rent = &Rent::from_account_info(rent_sysvar_info)?;
 
     if !realm_info.data_is_empty() {
@@ -84,6 +85,7 @@ pub fn process_create_realm(
         council_mint: council_token_mint_address,
         name: name.clone(),
         reserved: [0; 8],
+        authority: Some(*realm_authority_info.key),
     };
 
     create_and_serialize_account_signed::<Realm>(
