@@ -31,11 +31,20 @@ pub struct Realm {
     /// Council mint
     pub council_mint: Option<Pubkey>,
 
+    /// Realm authority. The authority must sign transactions which update the realm (ex. adding governance, setting council)
+    /// The authority can be transferer to Realm Governance and hence make the Realm self governed through proposals
+    /// Note: This field is not used yet. It's reserved for future versions
+    pub authority: Option<Pubkey>,
+
     /// Governance Realm name
     pub name: String,
 }
 
-impl AccountMaxSize for Realm {}
+impl AccountMaxSize for Realm {
+    fn get_max_size(&self) -> Option<usize> {
+        Some(self.name.len() + 111)
+    }
+}
 
 impl IsInitialized for Realm {
     fn is_initialized(&self) -> bool {
@@ -77,6 +86,38 @@ pub fn get_realm_data(
     get_account_data::<Realm>(realm_info, program_id)
 }
 
+/// Deserializes account and checks the given authority is Realm's authority
+pub fn get_realm_data_for_authority(
+    program_id: &Pubkey,
+    realm_info: &AccountInfo,
+    realm_authority: &Pubkey,
+) -> Result<Realm, ProgramError> {
+    let realm_data = get_account_data::<Realm>(realm_info, program_id)?;
+
+    if realm_data.authority.is_none() {
+        return Err(GovernanceError::RealmHasNoAuthority.into());
+    }
+
+    if realm_data.authority.unwrap() != *realm_authority {
+        return Err(GovernanceError::InvalidAuthorityForRealm.into());
+    }
+
+    Ok(realm_data)
+}
+
+/// Deserializes Ream account and asserts the given governing_token_mint is either Community or Council mint of the Realm
+pub fn get_realm_data_for_governing_token_mint(
+    program_id: &Pubkey,
+    realm_info: &AccountInfo,
+    governing_token_mint: &Pubkey,
+) -> Result<Realm, ProgramError> {
+    let realm_data = get_realm_data(program_id, realm_info)?;
+
+    realm_data.assert_is_valid_governing_token_mint(governing_token_mint)?;
+
+    Ok(realm_data)
+}
+
 /// Returns Realm PDA seeds
 pub fn get_realm_address_seeds(name: &str) -> [&[u8]; 2] {
     [PROGRAM_AUTHORITY_SEED, name.as_bytes()]
@@ -111,3 +152,25 @@ pub fn get_governing_token_holding_address(
     )
     .0
 }
+
+#[cfg(test)]
+mod test {
+
+    use super::*;
+
+    #[test]
+    fn test_max_size() {
+        let realm = Realm {
+            account_type: GovernanceAccountType::Realm,
+            community_mint: Pubkey::new_unique(),
+            reserved: [0; 8],
+            council_mint: Some(Pubkey::new_unique()),
+            authority: Some(Pubkey::new_unique()),
+            name: "test-realm".to_string(),
+        };
+
+        let size = realm.try_to_vec().unwrap().len();
+
+        assert_eq!(realm.get_max_size(), Some(size));
+    }
+}
