@@ -28,9 +28,9 @@ use spl_governance::{
         add_signatory, cancel_proposal, cast_vote, create_account_governance,
         create_mint_governance, create_program_governance, create_proposal, create_realm,
         create_token_governance, deposit_governing_tokens, execute_instruction, finalize_vote,
-        insert_instruction, relinquish_vote, remove_instruction, remove_signatory,
-        set_governance_config, set_governance_delegate, sign_off_proposal,
-        withdraw_governing_tokens, Vote,
+        flag_instruction_error, insert_instruction, relinquish_vote, remove_instruction,
+        remove_signatory, set_governance_config, set_governance_delegate, set_realm_authority,
+        sign_off_proposal, withdraw_governing_tokens, Vote,
     },
     processor::process_instruction,
     state::{
@@ -167,8 +167,11 @@ impl GovernanceProgramTest {
         )
         .await;
 
+        let realm_authority = Keypair::new();
+
         let create_realm_instruction = create_realm(
             &self.program_id,
+            &realm_authority.pubkey(),
             &community_token_mint_keypair.pubkey(),
             &self.context.payer.pubkey(),
             Some(council_token_mint_keypair.pubkey()),
@@ -185,6 +188,7 @@ impl GovernanceProgramTest {
             council_mint: Some(council_token_mint_keypair.pubkey()),
             name,
             reserved: [0; 8],
+            authority: Some(realm_authority.pubkey()),
         };
 
         RealmCookie {
@@ -196,6 +200,7 @@ impl GovernanceProgramTest {
 
             council_token_holding_account: Some(council_token_holding_address),
             council_mint_authority: Some(council_token_mint_authority),
+            realm_authority,
         }
     }
 
@@ -207,8 +212,11 @@ impl GovernanceProgramTest {
         let realm_address = get_realm_address(&self.program_id, &name);
         let council_mint = realm_cookie.account.council_mint.unwrap();
 
+        let realm_authority = Keypair::new();
+
         let create_realm_instruction = create_realm(
             &self.program_id,
+            &realm_authority.pubkey(),
             &realm_cookie.account.community_mint,
             &self.context.payer.pubkey(),
             Some(council_mint),
@@ -225,6 +233,7 @@ impl GovernanceProgramTest {
             council_mint: Some(council_mint),
             name,
             reserved: [0; 8],
+            authority: Some(realm_authority.pubkey()),
         };
 
         let community_token_holding_address = get_governing_token_holding_address(
@@ -247,6 +256,7 @@ impl GovernanceProgramTest {
             council_mint_authority: Some(clone_keypair(
                 realm_cookie.council_mint_authority.as_ref().unwrap(),
             )),
+            realm_authority,
         }
     }
 
@@ -313,6 +323,21 @@ impl GovernanceProgramTest {
         .await;
     }
 
+    #[allow(dead_code)]
+    pub async fn with_council_token_deposit_amount(
+        &mut self,
+        realm_cookie: &RealmCookie,
+        amount: u64,
+    ) -> TokeOwnerRecordCookie {
+        self.with_initial_governing_token_deposit(
+            &realm_cookie.address,
+            &realm_cookie.account.council_mint.unwrap(),
+            &realm_cookie.council_mint_authority.as_ref().unwrap(),
+            amount,
+        )
+        .await
+    }
+
     #[allow(dead_code)]
     pub async fn with_council_token_deposit(
         &mut self,
@@ -530,6 +555,45 @@ impl GovernanceProgramTest {
         .unwrap();
     }
 
+    #[allow(dead_code)]
+    pub async fn set_realm_authority(
+        &mut self,
+        realm_cookie: &RealmCookie,
+        new_realm_authority: &Option<Pubkey>,
+    ) -> Result<(), ProgramError> {
+        self.set_realm_authority_using_instruction(
+            realm_cookie,
+            new_realm_authority,
+            NopOverride,
+            None,
+        )
+        .await
+    }
+
+    #[allow(dead_code)]
+    pub async fn set_realm_authority_using_instruction<F: Fn(&mut Instruction)>(
+        &mut self,
+        realm_cookie: &RealmCookie,
+        new_realm_authority: &Option<Pubkey>,
+        instruction_override: F,
+        signers_override: Option<&[&Keypair]>,
+    ) -> Result<(), ProgramError> {
+        let mut set_realm_authority_ix = set_realm_authority(
+            &self.program_id,
+            &realm_cookie.address,
+            &realm_cookie.realm_authority.pubkey(),
+            new_realm_authority,
+        );
+
+        instruction_override(&mut set_realm_authority_ix);
+
+        let default_signers = &[&realm_cookie.realm_authority];
+        let signers = signers_override.unwrap_or(default_signers);
+
+        self.process_transaction(&[set_realm_authority_ix], Some(signers))
+            .await
+    }
+
     #[allow(dead_code)]
     pub async fn withdraw_community_tokens(
         &mut self,
@@ -642,7 +706,8 @@ impl GovernanceProgramTest {
 
     pub fn get_default_governance_config(&mut self) -> GovernanceConfig {
         GovernanceConfig {
-            min_tokens_to_create_proposal: 5,
+            min_community_tokens_to_create_proposal: 5,
+            min_council_tokens_to_create_proposal: 2,
             min_instruction_hold_up_time: 10,
             max_voting_time: 10,
             vote_threshold_percentage: VoteThresholdPercentage::YesVote(60),
@@ -811,9 +876,9 @@ impl GovernanceProgramTest {
         instruction_override(&mut create_program_governance_instruction);
 
         let default_signers = &[&governed_program_cookie.upgrade_authority];
-        let singers = signers_override.unwrap_or(default_signers);
+        let signers = signers_override.unwrap_or(default_signers);
 
-        self.process_transaction(&[create_program_governance_instruction], Some(singers))
+        self.process_transaction(&[create_program_governance_instruction], Some(signers))
             .await?;
 
         let account = Governance {
@@ -876,9 +941,9 @@ impl GovernanceProgramTest {
         instruction_override(&mut create_mint_governance_instruction);
 
         let default_signers = &[&governed_mint_cookie.mint_authority];
-        let singers = signers_override.unwrap_or(default_signers);
+        let signers = signers_override.unwrap_or(default_signers);
 
-        self.process_transaction(&[create_mint_governance_instruction], Some(singers))
+        self.process_transaction(&[create_mint_governance_instruction], Some(signers))
             .await?;
 
         let account = Governance {
@@ -941,9 +1006,9 @@ impl GovernanceProgramTest {
         instruction_override(&mut create_token_governance_instruction);
 
         let default_signers = &[&governed_token_cookie.token_owner];
-        let singers = signers_override.unwrap_or(default_signers);
+        let signers = signers_override.unwrap_or(default_signers);
 
-        self.process_transaction(&[create_token_governance_instruction], Some(singers))
+        self.process_transaction(&[create_token_governance_instruction], Some(signers))
             .await?;
 
         let account = Governance {
@@ -1021,7 +1086,7 @@ impl GovernanceProgramTest {
         let mut create_proposal_instruction = create_proposal(
             &self.program_id,
             &governance_cookie.address,
-            &token_owner_record_cookie.token_owner.pubkey(),
+            &token_owner_record_cookie.address,
             &governance_authority.pubkey(),
             &self.context.payer.pubkey(),
             &governance_cookie.account.realm,
@@ -1067,6 +1132,7 @@ impl GovernanceProgramTest {
             no_votes_count: 0,
 
             execution_flags: InstructionExecutionFlags::None,
+            governing_token_mint_vote_supply: None,
         };
 
         let proposal_address = get_proposal_address(
@@ -1159,17 +1225,36 @@ impl GovernanceProgramTest {
         proposal_cookie: &ProposalCookie,
         signatory_record_cookie: &SignatoryRecordCookie,
     ) -> Result<(), ProgramError> {
-        let sign_off_proposal_instruction = sign_off_proposal(
+        self.sign_off_proposal_using_instruction(
+            proposal_cookie,
+            signatory_record_cookie,
+            NopOverride,
+            None,
+        )
+        .await
+    }
+
+    #[allow(dead_code)]
+    pub async fn sign_off_proposal_using_instruction<F: Fn(&mut Instruction)>(
+        &mut self,
+        proposal_cookie: &ProposalCookie,
+        signatory_record_cookie: &SignatoryRecordCookie,
+        instruction_override: F,
+        signers_override: Option<&[&Keypair]>,
+    ) -> Result<(), ProgramError> {
+        let mut sign_off_proposal_ix = sign_off_proposal(
             &self.program_id,
             &proposal_cookie.address,
             &signatory_record_cookie.signatory.pubkey(),
         );
 
-        self.process_transaction(
-            &[sign_off_proposal_instruction],
-            Some(&[&signatory_record_cookie.signatory]),
-        )
-        .await?;
+        instruction_override(&mut sign_off_proposal_ix);
+
+        let default_signers = &[&signatory_record_cookie.signatory];
+        let signers = signers_override.unwrap_or(default_signers);
+
+        self.process_transaction(&[sign_off_proposal_ix], Some(signers))
+            .await?;
 
         Ok(())
     }
@@ -1605,6 +1690,27 @@ impl GovernanceProgramTest {
             .await
     }
 
+    #[allow(dead_code)]
+    pub async fn flag_instruction_error(
+        &mut self,
+        proposal_cookie: &ProposalCookie,
+        token_owner_record_cookie: &TokeOwnerRecordCookie,
+        proposal_instruction_cookie: &ProposalInstructionCookie,
+    ) -> Result<(), ProgramError> {
+        let governance_authority = token_owner_record_cookie.get_governance_authority();
+
+        let flag_instruction_error_ix = flag_instruction_error(
+            &self.program_id,
+            &proposal_cookie.address,
+            &proposal_cookie.account.token_owner_record,
+            &governance_authority.pubkey(),
+            &proposal_instruction_cookie.address,
+        );
+
+        self.process_transaction(&[flag_instruction_error_ix], Some(&[&governance_authority]))
+            .await
+    }
+
     #[allow(dead_code)]
     pub async fn get_token_owner_record_account(&mut self, address: &Pubkey) -> TokenOwnerRecord {
         self.get_borsh_account::<TokenOwnerRecord>(address).await
